C////////////////////////////////////////////////////////////////////
C/ Compiler switches used:
C/ No save of local variables
C/ Default integer size: 4 bytes
C
C
C****************************************************************************
C* This is a FORTRAN file that defines the routines for computing   
C* daily K indices by the FMI-method.                                        
C*                                                                           
C* There is only one routine that the user needs to call in order to compute 
C* K indices. That function is called 'K_index' and its definition is :      
C*                                                                           
C* SUBROUTINE K_index(X_data,Y_data,Arraysize,SampleStep,K9_limit,      
C*              long Longitude,long MissingData,long *K_table)               
C*                                                                           
C* The parameters are:                                                       
C*      X_data      Data array containing the INTEGER*4                      
C*                  X-component of the field values. The field values are    
C*                  stored as long's and may be whatever unit one uses (e.g. 
C*                  1 nT or 0.1 nT). If one normally uses decimal points     
C*                  (e.g. 49123.5) the values must be converted to long's    
C*                  (e.g. 491235) before the routine can be called.          
C*                  In order to compute the K indices for one day the FMI    
C*                  method requires data for previous day and next day.      
C*                  Therefore the X_data array must contain data exactly for 
C*                  three days (i.e if sampling rate is one minute the X_data
C*                  array must have 3*1440 = 4320 elements).
C*                  NOTE THAT THE DIMENSION STATEMENT FOR X_data IN THE 
C*                  CALLING ROUTINE MUST BE
C*                  INTEGER*4 X_data(0:ArraySize),Y_data(0:ArraySize)
C*      Y_data      Data array containing the INTEGER*4   
C*                  Y-component of the field values. SEE ABOVE                        
C*      ArraySize   INTEGER*4 giving the dimension size of X_data and Y_data
C*      SampleStep  INTEGER*4 Interval between successive data points in     
C*                  seconds (e.g. if minute values then SampleStep = 60).    
C*      K9_limit    INTEGER*4 K=9 limit for the particular observatory. Here
C*                  the unit is the same as used in the field values. (e.g if
C*                  the K=9 limit is 750 nT for the observatory and field
C*                  values are expressed with 0.1 nT accuracy then
C*                  K9_limit = 7500).    
C*      Longitude   This is the INTEGER*4 integral part of the longitude of
C*                  the observatory whose K indices are to be computed. The
C*                  FMI method uses this to determine the time of local
C*                  midnight. (e.g. Nurmijarvi longitude is 24.65, so
C*                  Longitude = 25). 
C*      MissingData INTEGER*4 Marker for missing data point (e.g. 999999).             
C*      K_table     INTEGER*4 table (array) where the computed eight K values
C*                  are returned. So one should define in his/her program an
C*                  array: INTEGER*4 K(0:7). If the program is unable to compute
C*                  a K value (due to missing data) a value -1 is substituted  
C*                  for that particular element.                             
C*                                                                           
C*****************************************************************************/
C*****************************************************************************/
C*                          Written by Lasse Hakkinen                        
C*          Finnish Meteorological Institute, Geophysics Department          
C*                              P.O.Box 503                                  
C*                      SF-00101, Helsinki, Finland                          
C*                      e-mail: Lasse.Hakkinen@fmi.fi   (internet)           
C*                      phone : (+358)-0-1929508                             
C*                      fax   : (+358)-0-1929539                             
C*                                                                           
C*                      version 1.0     29.9.1992                            
C****************************************************************************
C*
C****************************************************************************
C*       Retrogressively(!) Converted to FORTRAN by Phil McFadden
C*              Australian Geological Survey Organisation
C*                             GPO BOX 378
C*                          Canberra ACT 2601
C*                              AUSTRALIA
C*                       phone : -61-6-249 9612
C*                       fax   : -61-6-249 9986
C*                            December 1992
C*
C*              This particular conversion is not to FORTRAN90
C*               so there is no dynamic allocation of arrays!
C*****************************************************************************
                                



C*----------------------------------------------------------------------------
C* Initialize the global common block variables. Since a priori we don't
C* know the sampling rate and the size of the data arrays we should allocate
C* the space dynamically. However, since this is in FORTRAN(<90!), the arrays
C* have to be allocated at compile time. At run time, a quick check is
C* performed to see that sufficient space has been allocated. If there is
C* insufficient space a warning is issued and the program terminated.
C*----------------------------------------------------------------------------
	SUBROUTINE InitGlobals(Step,K9_limit,Longitude,MissingData)
	
	IMPLICIT NONE
	
	INTEGER*4 Step,K9_limit,Longitude,MissingData

C*---------------------------------------------------------------------------
C* Parameters of the FMI-method. These default values are best for several
C* observatories. Changing the values may in some cases result in slightly
C* better agreement between hand-scaled and computed K-indices.
C*---------------------------------------------------------------------------


	INTEGER*4 NightLength,DawnLength,DuskLength
C*	Night fitting, Dawn fitting, and Dusk fitting lengths in seconds

	INTEGER*4 HarmOrd
C* Order of harmonic fitting

	INTEGER*4 MaxGapLen
C* Maximum length of gap of missing data points
C* in seconds. Smaller gaps will be interpolated
C* Larger gaps will lead to missing K-values.

	REAL*8 KExponent
C*	Exponent in KExponent


	PARAMETER( NightLength = 90*60 )
	PARAMETER( DawnLength = 60*60 )
	PARAMETER( DuskLength = 60*60 )
	PARAMETER( HarmOrd = 5 )
	PARAMETER( MaxGapLen = 15*60 )
	PARAMETER( KExponent = 3.3 )




C*---------------------------------------------------------------------------
C* The global variables used in the computation of the K indices.
C*---------------------------------------------------------------------------
	INTEGER*4 Blocksize   ! Number of values in the 3-day data
	PARAMETER( BlockSize = 1440 )  ! 1440 = 24*3600/60
	
	INTEGER*4 X_temp(0:BlockSize)    !* Data array for X-comp.
	INTEGER*4 Y_temp(0:BlockSize)    !* Data array for Y-comp.
	INTEGER*4 X_harm(0:BlockSize)    !* Harmonic fitting array of X-curve
	INTEGER*4 Y_harm(0:BlockSize)    !* Harmonic fitting array of Y-curve
	COMMON/TEMPS/X_temp,Y_temp
	COMMON/HARMS/X_harm,Y_harm
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount       ! Number of datapoints in three days
	INTEGER*4 StepSize        ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	INTEGER*4 K_limit(0:8)    ! Array of limiting values of the K indices
	INTEGER*4 DefLength(0:23) ! Default fitting length for each hour
	INTEGER*4 K_length(0:9)   ! Fitting lengths for each K value
	INTEGER*4 FitLength(0:23) ! Computed fitting length for each hour
	INTEGER*4 X_mean(0:23)    ! Mean value for each hour over fitting area
	INTEGER*4 Y_mean(0:23)    ! Mean value for each hour over fitting area
	INTEGER*4 MaxGapLength
	
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	COMMON/LENGTHS/DefLength,K_length,FitLength
	COMMON/LIMITS/K_limit
	COMMON/MEANS/X_mean,Y_mean
	COMMON/GAP/MaxGapLength
	


	INTEGER*2 i              ! Dummy index
	INTEGER*4 DummyTable(0:23) ! Dummy table for adjusting DefLength table
	INTEGER*4 DefLimit(0:8)
	DATA DefLimit/75,150,300,600,1050,1800,3000,4950,7500/
                         !* Limits for different K indices (unit = 0.1 nT)
                         
	SAVE
		
	PointsInDay  = (24*3600)/Step
	DataCount    = 3*PointsInDay
	StepSize     = Step
	MissingValue = MissingData
	MaxGapLength = MaxGapLen
	HarmOrder    = HarmOrd
	IF( PointsInDay .GT. BlockSize ) THEN
		PRINT *, CHAR(7),CHAR(7)
		PRINT *, 'The arrays are dimensioned too small'
		PRINT *, 'The current dimension is ',Blocksize
		PRINT *, 'and you need a dimension of ',PointsInDay
		PRINT *, 'You must change the statement'
		PRINT *, 'PARAMETER( BlockSize = ',BlockSize,')'
		PRINT *, 'to (at minimum)'
		PRINT *, 'PARAMETER( BlockSize = ',PointsInDay,')'
		PRINT *, 'in EACH of the routines'
		PRINT *, 'SUBROUTINE InitGlobals'
		PRINT *, 'SUBROUTINE FillKTable'
		STOP
	ENDIF
	
C* Set the limit values for K indices
	DO i=0,8
		k_limit(i) = ( K9_limit*DefLimit(i) )/7500
	END DO
	
C* Set the default fitting lengths for each hour
	DO i=0,2
		DefLength(i)    = NightLength
		DefLength(i+3)  = DawnLength
		DefLength(i+18) = DuskLength
		DefLength(i+21) = NightLength
	END DO
	DO i=6,17
		DefLength(i) = 0
	END DO
	
C* Rotate the DefLength table to adjust the midnight
	DO i=0,23
		DummyTable(i) = DefLength( mod( (i+Longitude/15), 24))
	END DO
	DO i=0,23
		DefLength(i) = DummyTable(i)
	END DO
	
C* Compute the fitting lengths for different K-values
	K_length(0) = 0
	DO i=1,9
		K_length(i) = 60*EXP( KExponent*ALOG(float(i)) )
		! max = 18 hours
		if (K_length(i) > 18*3600) K_length(i) = 18*3600
	END DO
	
	RETURN
	END








C*----------------------------------------------------------------------------
C* Go through the three-day data and interpolate gaps whose lengths are      
C* smaller than MaxGapLength. If gaps are larger than this leave them        
C* untouched.  K index computation routine will notice the gaps and return a 
C* Missing K value for the particular three hour period.                     
C*----------------------------------------------------------------------------
	SUBROUTINE FillGaps(Data_array)
	IMPLICIT NONE
	INTEGER*4 Data_array(0:*)
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount      ! Number of datapoints in three days
	INTEGER*4 StepSize       ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	INTEGER*4 MaxGapLength
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	COMMON/GAP/MaxGapLength
	
	
	SAVE /V1/,/GAP/
	
	
	INTEGER*4 i,j         ! Indices for data points                  
	INTEGER*4 GapIndex   ! Index of the first missing data point    
	INTEGER*4 GapLength  ! Number of missing data points            
	INTEGER*4 Value1     ! Last non-missing data point before gap   
	INTEGER*4 Value2     ! First non-missing data point after gap   
	
	i = 0
	DO WHILE(i .LT. DataCount)     ! Go through all the three days
		!*--- Find the next missing value ---*/
		DO WHILE((i .LT. DataCount) .AND.
     1			 (Data_array(i) .NE. MissingValue))
			i=i+1
		END DO
		
		!*--- If a gap was found handle it ---*/
		IF(i .LT. DataCount) THEN
			GapIndex = i
			!* Find the last missing point */
			DO WHILE((i .LT. DataCount) .AND.
     1				 (Data_array(i) .EQ. MissingValue))
     				i=i+1
     			END DO
			
C*--- Interpolate the gap if possible. We cannot extrapolate ---*/
			IF( (GapIndex .GT. 0)
     1			 .AND. (i .LT. DataCount) ) THEN
				!*--- Now i is the index of first non-missing
				!*--- value and GapIndex is the index of
				!*--- first missing value ---*/
				GapLength = i-GapIndex
				!*--- Interpolate if the gap is small enough
				IF(GapLength*StepSize.LT.MaxGapLength)
     1					THEN
					Value1 = Data_array(GapIndex-1)
					Value2 = Data_array(i)
					DO j=1,GapLength
						Data_array(GapIndex) =
     1			Value1+(j*(Value2-Value1))/(GapLength+1)
						GapIndex=GapIndex+1
					END DO
 				ENDIF
 			 ENDIF
 		ENDIF
 	END DO
	RETURN
	END





C*----------------------------------------------------------------------------
C* Copy one day data from FromArray to ToArray                               
C*----------------------------------------------------------------------------
	SUBROUTINE CopyData(FromArray,ToArray)
	IMPLICIT NONE
	INTEGER*4 FromArray(0:*),ToArray(0:*)
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount      ! Number of datapoints in three days
	INTEGER*4 StepSize       ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	SAVE /V1/
	
	
	INTEGER*4 i
	
C* FromArray contains data for three days.
C* Select the middle one.
	DO i=0,(PointsInDay-1)
		ToArray(i) = FromArray(i+PointsInDay)
	END DO
	
	RETURN
	END






C*----------------------------------------------------------------------------
C* Compute the difference : DiffArray = Array1 - Array2.                     
C*----------------------------------------------------------------------------
	SUBROUTINE ComputeDiff(Array1,Array2,DiffArray)
	IMPLICIT NONE
	INTEGER*4 Array1(0:*),Array2(0:*),DiffArray(0:*)
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount      ! Number of datapoints in three days
	INTEGER*4 StepSize       ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	SAVE /V1/
	
	
	
	INTEGER*4 i,j
	
C* Array1 contains data for three days.     
C* Select the middle one.	
	DO i=0,(PointsInDay-1)
		j=i+PointsInDay
		IF( (Array1(j) .NE. MissingValue) .AND.
     1		    (Array2(i) .NE. MissingValue) ) THEN
			DiffArray(i) = Array1(j) - Array2(i)
            	ELSE
            		DiffArray(i) = MissingValue
 		ENDIF
 	END DO
 	
	RETURN
	END





C*----------------------------------------------------------------------------
C* Compute the difference between the maximum and minimum values in the three
C* hour part in the Data_array and determine the corresponding K index.      
C*----------------------------------------------------------------------------
	INTEGER*4 FUNCTION K_MaxMin(Data_array,HourIndex)
	IMPLICIT NONE
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount      ! Number of datapoints in three days
	INTEGER*4 StepSize       ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	INTEGER*4 K_limit(0:8)    ! Array of limiting values of the K indices
	
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	COMMON/LIMITS/K_limit
	SAVE /V1/,/LIMITS/

	INTEGER*4 Data_array(0:*),HourIndex
	INTEGER*4 i,j       ! Dummy index
	INTEGER*4 Max,Min   ! Maximum and minimum values
	
C* Find the maximum and minimum values */
	j = HourIndex*PointsInDay/8     !* Find the right 3-hour block*/
	Max = Data_array(j)
	Min = Max
C
C I HAVE CHANGED THIS FROM THE ORIGINAL BECAUSE I BELIEVE THE
C ORIGINAL IS INCORRECT. IT TESTS THE FIRST POINT TWICE AND
C TERMINATES BEFORE TESTING THE LAST POINT IN THE 3-HOUR SEQUENCE
C
	DO i=1,( (PointsInDay/8) - 1 ) !I think this is correct
C	DO i=0,( (PointsInDay/8) - 2 ) !I think this is wrong. Now confirmed
		IF(Data_array(j+i) .GT. Max) THEN
			Max = Data_array(j+i)
		ELSE IF(Data_array(j+i) .LT. Min) THEN
			Min = Data_array(j+i)
		ENDIF
	END DO
	

C* Find the K index corresponding to Max-Min */
	IF(Max .EQ. MissingValue) THEN
		K_MaxMin = -1       !* -1 if unable to compute K */
	ELSE      !* Go through the KLimit-table */
		i = 0
		DO WHILE( (i.LT.9) .AND. ( Max-Min .GT. K_limit(i) ) )
			i=i+1
		END DO
		K_MaxMin=i
	ENDIF
	RETURN
	END





C*----------------------------------------------------------------------------
C* Determine K indices by using the max-min method either directly to the    
C* original data (Subtract = false) or to the difference between original    
C* data and harmonic fitting (Subtract = true). The final K index is the     
C* larger one from the computed K's for the X and Y components.              
C*----------------------------------------------------------------------------
	SUBROUTINE FillKTable(K_table,X_data,Y_data,ArraySize,Subtract)
	IMPLICIT NONE
	
	INTEGER*4 K_MaxMin
	INTEGER*4 ArraySize,Subtract
	INTEGER*4 K_table(0:7),X_data(0:ArraySize-1),Y_data(0:ArraySize-1)
	
	INTEGER*4 Blocksize   ! Number of values in the 3-day data
	PARAMETER( BlockSize = 1440 )  ! 1440 = 24*3600/60
	
	INTEGER*4 X_temp(0:BlockSize)    !* Data array for X-comp.
	INTEGER*4 Y_temp(0:BlockSize)    !* Data array for Y-comp.
	INTEGER*4 X_harm(0:BlockSize)    !* Harmonic fitting array of X-curve
	INTEGER*4 Y_harm(0:BlockSize)    !* Harmonic fitting array of Y-curve
	COMMON/TEMPS/X_temp,Y_temp
	COMMON/HARMS/X_harm,Y_harm
	SAVE /TEMPS/,/HARMS/
	
	
	INTEGER*4 i         !* Index variable
	INTEGER*4 K_X(0:7)  !* Three hour K indices from X-component
	INTEGER*4 K_Y(0:7)  !* Three hour K indices from Y-component

C* Fill first the X_temp and Y_temp arrays */
	IF( Subtract .NE. 0 ) THEN
		CALL ComputeDiff(X_data,X_harm,X_temp)
		CALL ComputeDiff(Y_data,Y_harm,Y_temp)
	ELSE
		CALL CopyData(X_data,X_temp)
		CALL CopyData(Y_data,Y_temp)
	ENDIF

C* Now compute K indices for X and Y from max-min values */
C* for each three-hour interval and take the larger one  */
C* as the final K index.
	DO i=0,7
		K_X(i) = K_MaxMin(X_temp,i)
		K_Y(i) = K_MaxMin(Y_temp,i)
		IF( K_X(i) .GT. K_Y(i) ) THEN
			K_table(i) = K_X(i)
		ELSE
			K_table(i) = K_Y(i)
		ENDIF
	END DO
	
	RETURN
	END




C*----------------------------------------------------------------------------
C* Find the fitting lengths for each hour.                                   
C*----------------------------------------------------------------------------
	SUBROUTINE FindLengths(K_table)
	IMPLICIT NONE
	INTEGER*4 K_table(0:7)
	
	
	INTEGER*4 K_length(0:9)  ! Fitting lengths for each K value
	INTEGER*4 DefLength(0:23) ! Default fitting length for each hour
	INTEGER*4 FitLength(0:23) ! Computed fitting length for each hour
	
	COMMON/LENGTHS/DefLength,K_length,FitLength
	SAVE /LENGTHS/

	
	
	INTEGER*4 i
	
	DO i=0,23         !* go through all hours */
		IF( K_table(i/3) .LT. 0 )THEN
		    FitLength(i) = 30*60+DefLength(i)
		ELSE
		    FitLength(i) = 30*60+DefLength(i)+
     1					K_length(K_table(i/3))
		ENDIF
		
		IF( FitLength(i) .GT. 24*3600) THEN   !* not more than one day
		    FitLength(i) = 24*3600
		ENDIF
	END DO
	RETURN
	END




C*----------------------------------------------------------------------------
C* Compute mean values for each hour over the fitting interval.              
C*----------------------------------------------------------------------------
	INTEGER*4 FUNCTION ComputeMean(hour,Data_array)
	IMPLICIT NONE
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount       ! Number of datapoints in three days
	INTEGER*4 StepSize        ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	INTEGER*4 K_length(0:9)   ! Fitting lengths for each K value
	INTEGER*4 DefLength(0:23) ! Default fitting length for each hour
	INTEGER*4 FitLength(0:23) ! Computed fitting length for each hour
	
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	COMMON/LENGTHS/DefLength,K_length,FitLength
	SAVE /V1/,/LENGTHS/

	
	
	INTEGER*4 hour,Data_array(0:*)
	
	INTEGER*4 i,j           !* Dummy index
	INTEGER*4 MiddlePoint   !* Index of middle point of fitting interval
	INTEGER*4 WingLength    !* Number of datapoints before
	                        !* and after middle point
	INTEGER*4 Count         !* Number of datapoints in fitting interval
	INTEGER*4 Sum           !* Sum of field values
	
	MiddlePoint = PointsInDay+(60*(60*hour+30))/StepSize
	WingLength  = FitLength(hour)/StepSize
	j = MiddlePoint-WingLength
	Count = 2*WingLength+1
	Sum = 0
	
	DO i=0,(Count-1)
		IF( Data_array(i+j) .EQ. MissingValue ) THEN
			ComputeMean = MissingValue
			RETURN
		ELSE
			Sum = Sum + Data_array(i+j)
		ENDIF
	ENDDO
	ComputeMean = Sum/Count
	RETURN
	END






C*----------------------------------------------------------------------------
C* Interpolate the missing data values in the hourly mean value tables.      
C*----------------------------------------------------------------------------
	SUBROUTINE InterpolateMean(Mean_array)
	IMPLICIT NONE
	INTEGER*4 Mean_array(*)
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount       ! Number of datapoints in three days
	INTEGER*4 StepSize        ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	SAVE /V1/

	
	
	INTEGER*4 i,hour
	INTEGER*4 hour0
	INTEGER*4 hour1
	INTEGER*4 GapLength   !* Number of missing data points
	INTEGER*4 Value1      !* Last non-missing data point before gap
	INTEGER*4 Value2      !* First non-missing data point after gap
	LOGICAL  found        !* Boolean flag indicating missing data points
	
	hour0 = 0
	hour1 = 23
	
C* Extrapolate the first missing points */
	DO WHILE ( (hour0 .LT. 24) .AND.
     1	    (Mean_array(hour0) .EQ. MissingValue))
		hour0=hour0+1
	END DO
	
	IF(hour0 .LT. 24) THEN
		DO hour=0,(hour0-1)     !THERE WAS A BUG HERE INITIALLY
			Mean_array(hour) = Mean_array(hour0)
		END DO
	ENDIF
	
C* Extrapolate the last missing points */
	DO WHILE( (hour1 .GT. hour0) .AND.
     1	  (Mean_array(hour1) .EQ. MissingValue) )
	  hour1=hour1-1
	END DO
	
	DO hour=(hour1+1),23
		Mean_array(hour) = Mean_array(hour1)
	END DO
	
C* Interpolate the missing points between hour0 and hour1 */
C* Both hour0 and hour1 are hours where data exists       */
	DO WHILE(hour0 .LT. hour1)
10			hour0=hour0+1
			found = (Mean_array(hour0) .EQ. MissingValue)
			IF( (hour0 .LT. hour1)
     1			  .AND. ( .NOT. found ) ) GOTO 10
			IF(found) THEN
				hour = hour0      !* first missing hour */
				DO WHILE( (hour0 .LT.hour1) .AND.
     1				  (Mean_array(hour0).EQ.MissingValue))
					hour0=hour0+1
				END DO
				GapLength = hour0-hour
				Value1 = Mean_array(hour-1)
				Value2 = Mean_array(hour0)
				DO i=1,GapLength
	    Mean_array(hour) = Value1+(i*(Value2-Value1))/(GapLength+1)
				    hour=hour+1
				END DO
			ENDIF
	END DO
	
	RETURN
	END





C*----------------------------------------------------------------------------
C* Compute the Hour averages and interpolate missing hour values if needed.  
C*----------------------------------------------------------------------------
	SUBROUTINE ComputeHourAverages(X_data,Y_data,ArraySize)
	IMPLICIT NONE
	
	INTEGER*4 X_mean(0:23)    ! Mean value for each hour over fitting area
	INTEGER*4 Y_mean(0:23)    ! Mean value for each hour over fitting area
	
	COMMON/MEANS/X_mean,Y_mean
	SAVE /MEANS/

	
	
	INTEGER*4 ComputeMean
	INTEGER*4 ArraySize
	INTEGER*4 X_data(0:ArraySize-1),Y_data(0:ArraySize-1)
	
	INTEGER*4 hour

	DO hour=0,23
		X_mean(hour) = ComputeMean(hour,X_data)
		Y_mean(hour) = ComputeMean(hour,Y_data)
	END DO
C* Interpolate or extrapolate the possible gaps */
	CALL InterpolateMean(X_mean)
	CALL InterpolateMean(Y_mean)
	
	RETURN
	END





C*----------------------------------------------------------------------------
C* Compute the harmonic fit to the Hour-averages and put data values into    
C* X_harm- and Y_harm-tables.                                                
C*----------------------------------------------------------------------------
	SUBROUTINE ComputeHarmonicFit()
	IMPLICIT NONE
	
	REAL*8 PI
	PARAMETER( PI=3.141592654 )
	
	
	INTEGER*4 Blocksize   ! Number of values in the 3-day data
	PARAMETER( BlockSize = 1440 )  ! 1440 = 24*3600/60
	
	INTEGER*4 X_harm(0:BlockSize)    !* Harmonic fitting array of X-curve
	INTEGER*4 Y_harm(0:BlockSize)    !* Harmonic fitting array of Y-curve
	COMMON/HARMS/X_harm,Y_harm
	
	INTEGER*4 PointsInDay     ! Number of datapoints in one day
	INTEGER*4 DataCount      ! Number of datapoints in three days
	INTEGER*4 StepSize       ! Data sampling interval in seconds
	INTEGER*4 MissingValue    ! Marker for missing data point
	INTEGER*4 HarmOrder       ! Order of harmonic fitting
	INTEGER*4 X_mean(0:23)    ! Mean value for each hour over fitting area
	INTEGER*4 Y_mean(0:23)    ! Mean value for each hour over fitting area
	
	COMMON/V1/PointsInDay,DataCount,StepSize,MissingValue,HarmOrder
	COMMON/MEANS/X_mean,Y_mean
	SAVE /V1/,/MEANS/,/HARMS/
	
	
	
	INTEGER*4 i,k           !* Dummy indices */
	REAL*4 X_coeff
	REAL*4 Y_coeff
	REAL*4 ReX(0:12),ImX(0:12)
	REAL*4 ReY(0:12),ImY(0:12)
	REAL*4 si,co
	REAL*4 angle,angle2
	INTEGER*4  t,t0,t1

	t0 = 30*60         !* Middle point of first hour in seconds */
	t1 = 23*3600+t0    !* Middle point of last hour in seconds  */

C* Rotate the curve so that start and end points are equal */
	X_coeff = float(X_mean(23)-X_mean(0)) /(t1-t0)
	Y_coeff = float(Y_mean(23)-Y_mean(0)) /(t1-t0)
	t = t0
	DO i=0,23
		X_mean(i) = X_mean(i) - (X_coeff*(t-t0))
		Y_mean(i) = Y_mean(i) - (Y_coeff*(t-t0))
		t = t + 3600
	END DO

C* Compute the Fourier coefficients */
	DO i=0,HarmOrder
		ReX(i) = X_mean(0)
		ImX(i) = 0.0
		ReY(i) = Y_mean(0)
		ImY(i) = 0.0
		angle = -i*(2.0*PI/24.0)
		DO k=1,23
			si = sin(k*angle)
			co = cos(k*angle)
			ReX(i) = ReX(i) + X_mean(k)*co
			ImX(i) = ImX(i) + X_mean(k)*si
			ReY(i) = ReY(i) + Y_mean(k)*co
			ImY(i) = ImY(i) + Y_mean(k)*si
		END DO
	END DO

C* Compute the inverse fourier transform taking into account */
C* only terms up to HarmOrder.                               */
	t = 0
	angle = 2.0*PI*(23.0/24.0)/(t1-t0)
	DO i=0,(PointsInDay-1)
		X_harm(i) = ReX(0)
		Y_harm(i) = ReY(0)
		angle2 = (t-t0)*angle
		DO k=1,HarmOrder
			si = sin(k*angle2)
			co = cos(k*angle2)
			X_harm(i) = X_harm(i) + (2.0*(ReX(k)*co-ImX(k)*si))
			Y_harm(i) = Y_harm(i) + (2.0*(ReY(k)*co-ImY(k)*si))
		END DO
		!* Rotate the curve back */
		X_harm(i) = X_harm(i)/24.0+X_coeff*(t-t0)
		Y_harm(i) = Y_harm(i)/24.0+Y_coeff*(t-t0)
		t = t + StepSize
	END DO
	
	RETURN
	END





C*----------------------------------------------------------------------------
C* Compute the K indices for one day. This is the only function that the user
C* should ever need to call in this header file.                             
C*----------------------------------------------------------------------------
	SUBROUTINE K_index(X_data,Y_data,ArraySize,SampleStep,
     1		K9_limit,Longitude,MissingData,K_table)
	         
	         
	
	IMPLICIT NONE
	INTEGER*4 ArraySize
	INTEGER*4 X_data(0:ArraySize-1),Y_data(0:ArraySize-1)
	INTEGER*4 SampleStep,K9_limit,Longitude,MissingData
	INTEGER*4 K_table(0:7)
	INTEGER*4 i
	SAVE
	
C* Initialize the global variables */
	CALL InitGlobals(SampleStep,K9_limit,Longitude,MissingData)
	
	CALL FillGaps(X_data) ! Try to interpolate possible gaps in the data
	CALL FillGaps(Y_data)

C*** Step 1: Compute preliminary K indices by max-min method ***/
	CALL FillKTable(K_table,X_data,Y_data,ArraySize,0)
C* Here 0 implies not to subtract */
C* harmonic fitting from orig data*/

C*** Step 2: Compute the fitting lengths and find average values ***/
C*** for each hour                                               ***/
	CALL FindLengths(K_table)
	CALL ComputeHourAverages(X_data,Y_data,ArraySize)

C*** Step 3: Compute the harmonic fit to hour averages ***/
	CALL ComputeHarmonicFit()

C*** Step 4: Compute new K values from Original data - Harmonic fit
	CALL FillKTable(K_table,X_data,Y_data,ArraySize,1)
C* Here 1 implies to subtract     
C* harmonic fitting from orig data

C*** Step 5: Compute the fitting lengths and find average values ***/
C*** for each hour                                               ***/
	CALL FindLengths(K_table)
	CALL ComputeHourAverages(X_data,Y_data,ArraySize)
C*** Step 6: Compute the harmonic fit to hour averages ***/
	CALL ComputeHarmonicFit()

C*** Step 4: Compute final K values from Original data - Harmonic fit
	CALL FillKTable(K_table,X_data,Y_data,ArraySize,1)
		
	RETURN
	END
